@@ -8,6 +8,4 @@
     (gchar *)(rfm_void(RFM_MODULE_DIR, "icons", "mime_icon_get_global_xml_file"))
 # define ICON_get_filename_from_id(x) \
     (gchar *) (rfm_natural(RFM_MODULE_DIR, "icons", (void *)(x), "mime_icon_get_filename_from_id"))
-# define ICON_get_filename_from_basename(x) \
-    (gchar *) (rfm_natural(RFM_MODULE_DIR, "icons", (void *)(x), "mime_icon_get_filename_from_basename"))
 #endif
