@@ -13,4 +13,8 @@ rfm_cleanup_thumbnails(void *data);
 
 gchar * 
 rfm_content_icon_id_f (view_t *view_p, const gchar *path);
+
+gint 
+rfm_get_preview_image_size(void);
+
 #endif
