@@ -7,9 +7,6 @@ rodent_new_pathbar(void);
 void 
 rodent_new_gridview(widgets_t *widgets_p, const gchar *in_path);
 
-view_t *
-rodent_get_current_view ( GtkWidget * window);
-
 int 
 rodent_text_color(const population_t *population_p);
 
