@@ -189,7 +189,7 @@ mime_type (void *p, void *q) {
 	    goto extension_only;
 	}
     }
-    const gchar *type = (st_p)?mimeable_file (st_p):NULL;
+    const gchar *type = mimeable_file (st_p);
     if(type) {
 	put_mimetype_in_hash(file, type);
         NOOP ("MIME: stat mime_type(%s) -> %s\n", file, type);
@@ -278,7 +278,6 @@ mime_is_valid_command (void *p) {
     gchar **argv;
     if(!cmd_fmt)
         return GINT_TO_POINTER (FALSE);
-    // Skip to the first non environment set
     if(!g_shell_parse_argv (cmd_fmt, &argc, &argv, &error)) {
         gchar *msg = g_strcompress (error->message);
         DBG ("%s: %s\n", msg, cmd_fmt);
@@ -287,12 +286,17 @@ mime_is_valid_command (void *p) {
         return GINT_TO_POINTER (FALSE);
     }
     gchar **app = argv;
-    while (app && *app && strchr(*app, '='))app++;
-    if (!app || *app==NULL) {
-        g_strfreev (argv);
+    if (*app==NULL) {
         errno = ENOENT;
         return GINT_TO_POINTER (FALSE);
     }
+
+    // assume command is correct if environment is being set
+    if (strchr(*app, '=')){
+        g_strfreev (argv);
+        return GINT_TO_POINTER (TRUE);
+    }
+
     path = g_find_program_in_path (*app);
     if(!path) {
         gboolean direct_path = rfm_g_file_test (argv[0], G_FILE_TEST_EXISTS) ||
