@@ -43,6 +43,7 @@ typedef struct xmltree_t{
     gchar *repeat_names;
     GHashTable *echo_hash;
     GHashTable *attribute_hash;
+    GHashTable *editable_elements;
     gint text_activates_top_attribute;
 
     gboolean validated;
@@ -54,7 +55,15 @@ typedef struct xmltree_t{
 } xmltree_t;
 
 
+// XXX  XSD specific
+static gboolean choice_callback(Tag_item_t *src_tag, 
+        GtkTreeView *treeview, GtkTreeModel *treemodel, 
+        GtkTreeIter *sibling, gint flag);
+static void update_iconsXSD(GtkTreeModel *model, GtkTreeIter iter);
+static GdkPixbuf *get_attribute_pixbuf(Attribute_item_t *row_attribute);
+static Tag_item_t *set_attribute_colorXSD(GtkTreeModel *treemodel, GtkTreeIter *iter);
 #include "xmltree.i"
+#include "xsdtree.i"
 
 G_MODULE_EXPORT
 void *text_activates_top_attribute(xmltree_t *xmltree_p, void *q){
@@ -75,6 +84,8 @@ void *xmltree_new(void){
     }
     memset(xmltree_p, 0, sizeof(xmltree_t));
     xmltree_p->attribute_hash = g_hash_table_new(g_str_hash, g_str_equal);
+    xmltree_p->echo_hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+    xmltree_p->editable_elements = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
     return (void *)xmltree_p; 
 }
 
@@ -89,6 +100,7 @@ void *xmltree_free(xmltree_t *xmltree_p){
     g_free(xmltree_p->repeat_names);
     g_hash_table_destroy(xmltree_p->attribute_hash);
     g_hash_table_destroy(xmltree_p->echo_hash);
+    g_hash_table_destroy(xmltree_p->editable_elements);
     g_free(xmltree_p);
     return NULL;
 }
@@ -315,8 +327,10 @@ void * xmltree_run(xmltree_t *xmltree_p){
 	    G_TYPE_STRING,   // attribute (string)
 	    G_TYPE_STRING,   // value (string)
 	    G_TYPE_STRING,    // prefix (string)
-	    G_TYPE_STRING    // color format
-	    //GDK_TYPE_COLOR    // color format
+	    GDK_TYPE_PIXBUF, // button column
+	    G_TYPE_STRING,    // color format
+            G_TYPE_INT         // flags
+
 	));
 	    // G_TYPE_BOXED, // this might be useful later on
 	    // G_TYPE_OBJECT,// and this one too...
@@ -331,8 +345,10 @@ void * xmltree_run(xmltree_t *xmltree_p){
 	    G_TYPE_STRING,   // attribute (string)
 	    G_TYPE_STRING,   // value (string)
 	    G_TYPE_STRING,    // prefix (string)
-	    G_TYPE_STRING    // color format
-	    //GDK_TYPE_COLOR    // color format
+	    GDK_TYPE_PIXBUF, // button column
+	    G_TYPE_STRING,    // color format
+            G_TYPE_INT         // flags
+
 	));
 
     if (!GTK_IS_TREE_MODEL(model)){
@@ -353,6 +369,8 @@ void * xmltree_run(xmltree_t *xmltree_p){
 
     gtk_tree_model_foreach(model, switch_colors, NULL);
 
+
+
     // Container for the graphic treeview.
     GtkWidget *treebox = build_treeview_box(treeview);
 
@@ -522,4 +540,219 @@ xmltree_attribute_item_add(
     return retval;
 }
 
+/////////////////////////////////////////////////////////////////////////////////////////////
+#include <libxml/xmlschemas.h>
+
+static const gchar *
+validate_schema(gchar *schema_file){
+    xmlDocPtr schema_doc = xmlReadFile(schema_file, NULL, XML_PARSE_NONET);
+    if (schema_doc == NULL) return "Cannot read schema file";
+    xmlSchemaParserCtxtPtr validationCtxt = xmlSchemaNewDocParserCtxt(schema_doc);
+    //xmlSchemaValidCtxtPtr validationCtxt = xmlSchemaNewValidCtxt (NULL);
+    if (validationCtxt == NULL) {
+	// Cannot create a validation context for the schema
+	return ("Failed to create the validation context.\n");
+    }
+
+    xmlSchemaPtr schema = xmlSchemaParse(validationCtxt);
+    if (schema == NULL) {
+        /* the schema itself is not valid */
+        xmlSchemaFreeParserCtxt(validationCtxt);
+	return ("schema itself is not valid\n");
+    }
+
+    xmlSchemaValidCtxtPtr valid_ctxt = xmlSchemaNewValidCtxt(schema);
+    if (valid_ctxt == NULL) {
+        xmlSchemaFree(schema);
+        xmlSchemaFreeParserCtxt(validationCtxt);
+        return ("unable to create a validation context for the schema\n");
+    }
+    return NULL;
+}
+
+static void
+validate_callbackXSD(GtkButton *b, void *data){
+    gchar *schema_file = data;
+    if (validate_schema(schema_file)==NULL){
+        // greenball somewhere
+        rfm_set_bin_image(GTK_WIDGET(b), "xffm/emblem_greenball", SIZE_BUTTON);
+    } else {
+        // redball somewhere
+        rfm_set_bin_image(GTK_WIDGET(b), "xffm/emblem_redball", SIZE_BUTTON);
+    }
+}
+
+
+G_MODULE_EXPORT
+void * xsdtree_run(xmltree_t *xmltree_p){
+    if (!xmltree_p) return NULL;
+
+    GError *error=NULL;
+    // FIXME: enter with dialog if null
+
+    if (g_file_test(xmltree_p->xml_path, G_FILE_TEST_EXISTS)){
+	xmltree_p->Tag_p =tag_new_from_file(xmltree_p->xml_path, &error);
+    } else {
+	xmltree_p->Tag_p = tag_new();
+    }
+
+
+    // Create treemodels
+    GtkTreeModel *schemamodel= GTK_TREE_MODEL(gtk_tree_store_new (H_TREE_COLUMNS,
+	    G_TYPE_POINTER,   // tag_item_p or parent's tag_item_p
+	    G_TYPE_POINTER,   // attribute_item_p if applicable.
+	    GDK_TYPE_PIXBUF, // pretty stuff
+	    G_TYPE_STRING,   // tag
+	    G_TYPE_STRING,   // attribute (string)
+	    G_TYPE_STRING,   // value (string)
+	    G_TYPE_STRING,    // prefix (string)
+	    GDK_TYPE_PIXBUF, // button column
+	    G_TYPE_STRING,    // color format
+            G_TYPE_INT         // flags
+	));
+	    // G_TYPE_BOXED, // this might be useful later on
+	    // G_TYPE_OBJECT,// and this one too...
+    g_object_set_data(G_OBJECT(schemamodel), "xmltree_p", xmltree_p);
+
+    GtkTreeModel *model= GTK_TREE_MODEL(gtk_tree_store_new (H_TREE_COLUMNS,
+	    G_TYPE_POINTER,   // tag_item_p or parent's tag_item_p
+	    G_TYPE_POINTER,   // attribute_item_p if applicable.
+	    GDK_TYPE_PIXBUF, // pretty stuff
+	    G_TYPE_STRING,   // tag
+	    G_TYPE_STRING,   // attribute (string)
+	    G_TYPE_STRING,   // value (string)
+	    G_TYPE_STRING,    // prefix (string)
+	    GDK_TYPE_PIXBUF, // button column
+	    G_TYPE_STRING,    // color format
+            G_TYPE_INT         // flags
+	));
+    GtkTreeModel *filtermodel = gtk_tree_model_filter_new(model, NULL);
+
+    if (!GTK_IS_TREE_MODEL(model)){
+	g_error("build_treemodel(): cannot create tree model!");
+    }
+    g_object_set_data(G_OBJECT(model), "xmltree_p", xmltree_p);
+    GtkWidget *treebox=NULL;
+     
+
+    GtkWidget *schemabox = NULL;
+    // Build schema treeview
+    if (xmltree_p->Tag_p) {
+	//Create schema tag
+	populate_tree_model_from_tag(xmltree_p->Tag_p, schemamodel, &error);
+	// Schema on the left.
+	GtkTreeView *schemaview = build_treeview(schemamodel);
+	g_object_set_data(G_OBJECT(schemaview), "xmltree_p", xmltree_p);
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(schemaview), FALSE);
+	// Container for the graphic schemaview.
+	schemabox = build_treeview_box(schemaview);
+    }
+
+    // Create a window for the treeview container.
+    GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    g_object_set_data(G_OBJECT(window), "xmltree_p", xmltree_p);
+    g_object_set_data(G_OBJECT(window), "Tag_p", xmltree_p->Tag_p);
+    g_object_set_data(G_OBJECT(model), "Tag_p", xmltree_p->Tag_p);
+    g_object_set_data(G_OBJECT(window), "model", model);
+ 
+    GtkTreeView *treeview;
+    // Create schema generated xml tag
+    Tag_t *xml_Tag_p = tag_new_from_schema_tag(xmltree_p, &error);
+    g_object_set_data(G_OBJECT(window), "xml_Tag_p", xml_Tag_p);
+    g_object_set_data(G_OBJECT(model), "xml_Tag_p", xml_Tag_p);
+
+    if (xml_Tag_p) {
+        populate_tree_model_from_tag(xml_Tag_p, model, &error);
+        // Build a graphic treeview.
+        // Xml to the right
+        treeview = build_treeview(filtermodel);
+        g_object_set_data(G_OBJECT(treeview), "filter_model", filtermodel);
+        gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(filtermodel), 
+                show_visible_row, NULL, NULL);
+        g_object_set_data(G_OBJECT(treeview), "xmltree_p", xmltree_p);
+        GtkTreePath *ipath = gtk_tree_path_new_from_string("0");
+        gtk_tree_view_expand_row(treeview, ipath, FALSE);
+        gtk_tree_path_free(ipath);
+        g_object_set_data(G_OBJECT(treeview), "Tag_p", xml_Tag_p);
+        g_object_set_data(G_OBJECT(model), "Tag_p", xml_Tag_p);
+//        g_object_set_data(G_OBJECT(model), "Tag_p", xmltree_p->Tag_p);
+
+        //gtk_tree_model_foreach(model, switch_colors, NULL);
+        gtk_tree_model_foreach(model, switch_colorsXSD, NULL);
+
+        // Container for the graphic treeview.
+        treebox = build_treeview_box(treeview);
+        // Create a shortcut reference.
+        g_object_set_data(G_OBJECT(treeview), "parent_window", window);
+        g_signal_connect(treeview, "key-press-event", G_CALLBACK(treeview_key), NULL);   
+    }
+
+    
+    const gchar *window_title = xmltree_p->window_title;
+
+
+
+    if (window_title) gtk_window_set_title (GTK_WINDOW (window), window_title);
+    gtk_window_set_default_size (GTK_WINDOW (window), 640, 480);
+    GtkWidget *vbox = rfm_vbox_new (FALSE, 0);
+    gtk_container_add (GTK_CONTAINER (window), vbox);
+
+    // Add a hpane
+    //GtkWidget *hpane = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
+    GtkWidget *hpane = rfm_hpaned_new();
+    gtk_box_pack_start (GTK_BOX (vbox), hpane, TRUE, TRUE, 3);
+    //gtk_container_add (GTK_CONTAINER (vbox), hpane);
+    if (schemabox) {
+	gtk_paned_add1 (GTK_PANED(hpane), schemabox);
+	if (treebox) gtk_paned_add2 (GTK_PANED(hpane), treebox);
+    } else {
+	if (treebox) gtk_paned_add1 (GTK_PANED(hpane), treebox);
+    }
+
+    GtkWidget * hbox = rfm_hbox_new (TRUE, 0);
+    GtkWidget *b;
+
+    b=rfm_dialog_button("xffm/emblem_blueball", _("Validate"));
+    g_signal_connect(G_OBJECT(b), "clicked", G_CALLBACK (validate_callbackXSD), xmltree_p->xml_path);
+    gtk_box_pack_start (GTK_BOX (hbox), b, TRUE, FALSE, 3);
+
+
+    if (xmltree_p->editable_attributes && *(xmltree_p->editable_attributes)) {
+        b=rfm_dialog_button("xffm/stock_save", _("Save"));
+        g_object_set_data(G_OBJECT(b), "callback", xsd_edit_save);
+        g_object_set_data(G_OBJECT(b), "window", window);
+        g_signal_connect(G_OBJECT(b), "clicked", G_CALLBACK (xsd_edit_save), window);
+        gtk_box_pack_start (GTK_BOX (hbox), b, TRUE, FALSE, 3);
+    }
+
+    b=rfm_dialog_button("xffm/stock_window-close", _("Close"));
+
+
+    g_object_set_data(G_OBJECT(b), "callback", xml_edit_destroy);
+    g_object_set_data(G_OBJECT(b), "window", window);
+    g_signal_connect(G_OBJECT(b), "clicked", G_CALLBACK (xml_edit_destroy), window);
+    gtk_box_pack_start (GTK_BOX (hbox), b, TRUE, FALSE, 3);
+
+
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 3);
+  
+
+    g_signal_connect(G_OBJECT(window), "destroy-event", G_CALLBACK (on_destroy_k), NULL);
+    g_signal_connect(G_OBJECT(window), "delete-event", G_CALLBACK (on_destroy_k), NULL);
+
+ 
+    gtk_widget_show_all(window);
+    gtk_paned_set_position(GTK_PANED(hpane), 0); 
+
+    // Event loop.
+    gtk_main();
+    return NULL;
+} 
+
+G_MODULE_EXPORT
+void *
+xsdtree_show_hidden(void *state){
+    show_hidden = GPOINTER_TO_INT(state);
+    return NULL;
+}
 
