@@ -722,7 +722,6 @@ pixbuf_from_file_f(void *data){
     gchar *path = arg[0];
     gint width = GPOINTER_TO_INT(arg[1]);
     gint height = GPOINTER_TO_INT(arg[2]);
-    g_free(arg);
     GError *error = NULL;
     GdkPixbuf *pixbuf = NULL;
     if (width < 0) {
@@ -730,6 +729,10 @@ pixbuf_from_file_f(void *data){
     } else {
 	pixbuf = gdk_pixbuf_new_from_file_at_size (path, width, height, &error);
     }
+    // hmmm... from the scale_simple line below, it seems that the above two
+    //         functions will do a g_object_ref on the returned pixbuf...
+
+
     // Gdkpixbuf Bug workaround 
     // (necessary for GTK-2, still necessary in GTK-3.8)
     // xpm icons not resized. Need the extra scale_simple. 
@@ -749,7 +752,6 @@ pixbuf_from_file_f(void *data){
 	    DBG ("pixbuf_from_file() %s:%s\n", error->message, path);
 	    g_error_free (error);
     }
-    g_free (path);
     return pixbuf;
 }
 
@@ -758,10 +760,10 @@ static pthread_mutex_t pixbuf_mutex = PTHREAD_MUTEX_INITIALIZER;
 GdkPixbuf *
 rfm_pixbuf_new_from_file (const gchar *path, gint width, gint height){
     if (!path) return NULL;
+    if (!g_file_test(path, G_FILE_TEST_EXISTS)) return NULL;
     GdkPixbuf *pixbuf;
-    void **arg =(void **)malloc(3*sizeof(void *));
-    if (!arg) g_error("malloc: %s\n", strerror(errno));
-    arg[0] = g_strdup(path);
+    void *arg[3];
+    arg[0] = (void *)path;
     arg[1] = GINT_TO_POINTER(width);
     arg[2] = GINT_TO_POINTER(height);
 #if 1
@@ -783,12 +785,9 @@ rfm_pixbuf_new_from_file (const gchar *path, gint width, gint height){
 
 #else
     // This sends everything to the gtk thread...
-    if (rfm_get_gtk_thread() != g_thread_self()){
 	pixbuf = rfm_context_function(pixbuf_from_file_f, arg);
-    } else {
-	pixbuf = pixbuf_from_file_f(arg);
-    }
 #endif
+
     return pixbuf;
 }
 
@@ -1241,21 +1240,19 @@ void rfm_free_lite_hash(void){
 static void *
 get_pixbuf_f(void *data){
     void **arg=data;
-    gchar *key = arg[0];
+    const gchar *key = (const gchar *)arg[0];
     gint size = GPOINTER_TO_INT(arg[1]);
-    g_free(arg);
     GdkPixbuf *pixbuf = get_pixbuf(key, size, FALSE); //refs
-    g_free(key);
     return pixbuf;
 }
 
 GdkPixbuf *
 rfm_get_pixbuf (const gchar * key, gint size) {
-    void **arg = (void **)malloc(2*sizeof(void *));
-    if (!arg) g_error("malloc: %s\n", strerror(errno));
-    arg[0] = g_strdup(key);
+    void *arg[2];
+    arg[0] = (void *)key;
     arg[1] = GINT_TO_POINTER(size);
-    return rfm_context_function(get_pixbuf_f, arg);
+    GdkPixbuf *pixbuf = rfm_context_function(get_pixbuf_f, arg);
+    return pixbuf;
 }
 
 // find in hash : refs
