@@ -30,8 +30,17 @@
 
 static GtkAllocation rfm_allocation;
 GtkAllocation *rfm_get_allocation(void){return &rfm_allocation;}
-void rfm_set_allocation(GtkAllocation *allocation){
-    memcpy(&rfm_allocation, allocation, sizeof(GtkAllocation));
+void rfm_set_allocation(void){
+    rfm_global_t *rfm_global_p = rfm_global();
+    gtk_widget_get_allocation (rfm_global_p->window,&rfm_allocation);
+   // nested in hpane...
+   GtkWidget *hpane = g_object_get_data(G_OBJECT(rfm_global_p->window), "hpane");
+   if (hpane){
+        NOOP(stderr, "signal_on_size_paper(): we are nested in an hpane...%d\n",
+                gtk_paned_get_position (GTK_PANED(hpane)));
+        rfm_allocation.width -= gtk_paned_get_position (GTK_PANED(hpane));
+    }   
+    //memcpy(&rfm_allocation, &allocation, sizeof(GtkAllocation));
     return;
 }
 
@@ -324,6 +333,56 @@ gint rfm_layout_get_cellwidth(view_t *view_p){
     return view_layout_p->cellwidth;
 }
 
+static void *
+get_drawable_geometry_f(void *data){
+    void **arg = data;
+    Drawable drawable = GPOINTER_TO_INT(arg[0]);
+    gint *x = (gint *)arg[1];
+    gint *y = (gint *)arg[2];
+    gint *w = (gint *)arg[3];
+    gint *h = (gint *)arg[4];
+    gint *d = (gint *)arg[5];
+    
+    rfm_global_t *rfm_global_p = rfm_global();
+    if (rfm_get_gtk_thread() != g_thread_self()){
+	g_error("rfm_get_drawable_geometry is a main thread function\n");
+    }
+    gint x_return, y_return;
+    guint w_return, h_return, d_return, border_return;
+    Window root_return;
+    Display *display;
+    if (rfm_global_p) display = rfm_global_p->Xdisplay;
+    else display = gdk_x11_display_get_xdisplay(gdk_display_get_default());
+
+    XGetGeometry(display, drawable, &root_return,
+	    &x_return, &y_return, 
+	    &w_return, &h_return, 
+	    &border_return, 
+	    &d_return);
+    if (x) *x = x_return;
+    if (y) *y = y_return;
+    if (w) *w = w_return;
+    if (h) *h = h_return;
+    if (d) *d = d_return;
+    return NULL;
+}
+
+void rfm_get_drawable_geometry(Drawable drawable, 
+	gint *x, gint *y, gint *w, gint *h, gint *d){
+    void *arg[]={
+        GINT_TO_POINTER(drawable),
+        (void *) x,
+        (void *) y,
+        (void *) w,
+        (void *) h,
+        (void *) d,
+        NULL
+    };
+    rfm_context_function(get_drawable_geometry_f, arg);
+    return;
+
+}
+#if 0
 void rfm_get_drawable_geometry(Drawable drawable, 
 	gint *x, gint *y, gint *w, gint *h, gint *d){
     rfm_global_t *rfm_global_p = rfm_global();
@@ -349,6 +408,7 @@ void rfm_get_drawable_geometry(Drawable drawable,
     if (d) *d = d_return;
     return;
 }
+#endif
 
 gint
 rfm_get_default_size(void){
@@ -389,7 +449,7 @@ static
 view_preferences_t *
 get_view_preferences_path (gint type, const gchar * path) {
     NOOP ("rodent_get_view_preferences()\n");
-    DBHashTable *preferences;
+    DBHashTable *preferences=NULL;
     const gchar *key;
     GString *gs;
     gchar *f;
@@ -400,7 +460,12 @@ get_view_preferences_path (gint type, const gchar * path) {
     }
         
     TRACE("opening %s...\n",f); 
-    preferences = dbh_new (f, NULL, DBH_READ_ONLY|DBH_THREAD_SAFE|DBH_PARALLEL_SAFE);
+    FILE *f_p = fopen(f,"r");
+    if (f_p){
+        fclose(f_p);
+        preferences = dbh_new (f, NULL, DBH_READ_ONLY|DBH_PARALLEL_SAFE|DBH_THREAD_SAFE);
+    }
+
     TRACE("open %s.\n",f); 
  
     if(!preferences) {
@@ -462,7 +527,7 @@ rfm_get_view_preferences (gint type, record_entry_t *target_en) {
 void
 rfm_save_view_preferences (view_t * view_p, record_entry_t *target_en) {
     view_preferences_t iconview_preferences;
-    DBHashTable *preferences;
+    DBHashTable *preferences=NULL;
     GString *gs;
     gchar *f;
     const gchar *key=NULL;
@@ -494,7 +559,11 @@ rfm_save_view_preferences (view_t * view_p, record_entry_t *target_en) {
     
 
     TRACE("opening %s...\n",f); 
-    preferences = dbh_new (f, NULL, DBH_PARALLEL_SAFE|DBH_THREAD_SAFE);
+    FILE *f_p = fopen(f,"r");
+    if (f_p){
+        fclose(f_p);
+        preferences = dbh_new (f, NULL, DBH_PARALLEL_SAFE|DBH_THREAD_SAFE);
+    }
     if(!preferences) {
         NOOP ("creating file: %s", f);
 	tmp_f =g_strdup_printf("%s-%d", f, (gint)getpid());
@@ -682,6 +751,11 @@ rfm_layout_pango_layout_setup(view_t *view_p){
 
 void 
 rfm_layout_get_vpane_allocation(view_t *view_p, GdkRectangle *rect){
+    if (!view_p){
+        memset(rect, 0, sizeof(GdkRectangle));
+        DBG("rfm_layout_get_vpane_allocation(): view_p cannot be null.\n");
+        return;
+    }
     view_layout_t *view_layout_p = &(view_p->view_layout);
     if (view_p->flags.type == DESKVIEW_TYPE){
 	rect->x = rect->y = 0;
@@ -832,6 +906,8 @@ layout_reset_columns(view_t *view_p){
 	    view_layout_p->max_elements = view_layout_p->grid_rows * view_layout_p->grid_columns;
 	}
     } else  {// (iconview)
+
+
 	view_layout_p->grid_columns = ((double)width / view_layout_p->cellwidth) + 0.5;
 	view_layout_p->grid_rows = (view_layout_p->max_elements) / view_layout_p->grid_columns;
 	if(view_layout_p->max_elements % view_layout_p->grid_columns)
@@ -1051,7 +1127,7 @@ rfm_get_population_label_rect(view_t * view_p, const population_t * population_p
     rect->height=0;
     rect->width=0;
     if(population_p->layout) {
-	rect->height += population_p->logical_rect.height;
+	rect->height = population_p->logical_rect.height;
 	rect->width = population_p->logical_rect.width;
     }
     gint x_offset = (view_layout_p->icon_size >= SMALL_ICON_SIZE)?
@@ -1112,13 +1188,20 @@ rfm_get_population_label_rect_full(view_t * view_p, const population_t * populat
 	return FALSE;
     }
     view_layout_t *view_layout_p = &(view_p->view_layout);
-    rect->width = population_p->logical_rect_full.width+4;
-    rect->height = population_p->logical_rect_full.height+4;
-    rect->y = label_rect.y+2;
+
+    rect->width = population_p->logical_rect_full.width;
+    rect->height = population_p->logical_rect_full.height;
+    rect->y = label_rect.y;
+
     // with texts under the icons, center the saturated text.
     if (view_layout_p->icon_size >= SMALL_ICON_SIZE) {
 	gint center = item_rect.x + (item_rect.width/2);
 	rect->x = center - (rect->width/2);
+	// bigger bg square for non details...
+	rect->width += 4;
+	rect->height += 4;
+	rect->y += 2;
+	
     } else {
 	// texts to the right of the icon, use label.x
 	rect->x = label_rect.x;
@@ -1141,42 +1224,8 @@ rfm_get_population_label_rect_full(view_t * view_p, const population_t * populat
     }
     // correct for underlap
     if (rect->x < MARGIN_LEFT(view_p)) rect->x = MARGIN_LEFT(view_p);
-    //if (rect->x < 0) rect->x = 0;
- 
-#if 0
-    gint offset = 3;
-    if (ICON_SIZE(view_p) >= SMALL_ICON_SIZE &&
-		    population_p->logical_rect_full.width > CELLWIDTH(view_p)) {
-	    GdkRectangle item_rect;
-	    if (!rfm_get_population_rect(view_p, population_p, &item_rect)) return;
-	offset = abs(label_rect.x - item_rect.x) + 5;
-    }
-#endif
     rfm_population_read_unlock(view_p, "rfm_get_population_label_rect_full");
     return TRUE;
 }
-#if 0
-layout_full_dimensions(view_t * view_p,
-	const population_t * population_p,
-	gint *width, gint *height){
-
-	if (!view_p || ! population_p) {
-		g_error("incorrect function call: layout_full_dimensions()");
-		return;
-	}
-    gint offset = 3;
-   	if (ICON_SIZE(view_p) >= SMALL_ICON_SIZE &&
-			population_p->logical_rect_full.width > CELLWIDTH(view_p)) {
-		GdkRectangle item_rect;
-		GdkRectangle label_rect;
-		if (!rfm_get_population_label_rect(view_p, population_p, &label_rect)) return;
-		if (!rfm_get_population_rect(view_p, population_p, &item_rect)) return;
-	    offset = abs(label_rect.x - item_rect.x) + 5;
-	}
-	// XXX logical_rect_full.width is not always truthful...
-	if (width) *width =  offset + population_p->logical_rect_full.width;
-	if (height) *height =  population_p->logical_rect_full.height;
-}
-#endif
 ////////////////////////////////////////////////////////////////////////////////////////////
 
