@@ -31,6 +31,44 @@
 #include "rfm_modules.h"
 #include "primary-misc.i"
 
+static void
+checkmenuitem_toggle(GtkWidget *menuitem, void *data){
+    gboolean state = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM (menuitem));
+    const gchar *markup_id = (state)?"on_label":"off_label";
+    GtkWidget *child = gtk_bin_get_child (GTK_BIN (menuitem));
+    gtk_label_set_markup (GTK_LABEL (child), 
+            (const gchar *)(g_object_get_data(G_OBJECT(menuitem), markup_id)));
+    gtk_label_set_markup (GTK_LABEL (child), 
+            (const gchar *)(g_object_get_data(G_OBJECT(menuitem), markup_id)));
+
+}
+
+GtkWidget *
+rfm_create_checkmenuitem(const gchar *label, gboolean state){
+    GtkWidget *menuitem = gtk_check_menu_item_new_with_label ("");
+    GtkWidget *child = gtk_bin_get_child (GTK_BIN (menuitem));
+    const gchar *markup_id = (state)?"on_label":"off_label";
+    g_object_set_data(G_OBJECT(menuitem), "on_label", g_strdup_printf("<b>%s</b>", label));
+    g_object_set_data(G_OBJECT(menuitem), "off_label", g_strdup_printf("<i>%s</i>", label));
+    
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM (menuitem), state);
+    checkmenuitem_toggle(menuitem, NULL);
+    g_signal_connect(G_OBJECT (menuitem), "toggled", G_CALLBACK (checkmenuitem_toggle), NULL);
+    return menuitem;
+}
+
+GtkWidget *
+rfm_create_radiomenuitem(const gchar *label, GSList *group){
+    GtkWidget *menuitem = gtk_radio_menu_item_new_with_mnemonic (group, label);
+    GtkWidget *child = gtk_bin_get_child (GTK_BIN (menuitem));
+    g_object_set_data(G_OBJECT(menuitem), "on_label", g_strdup_printf("<b>%s</b>", label));
+    g_object_set_data(G_OBJECT(menuitem), "off_label", g_strdup_printf("<i>%s</i>", label));
+    
+    checkmenuitem_toggle(menuitem, NULL);
+    g_signal_connect(G_OBJECT (menuitem), "toggled", G_CALLBACK (checkmenuitem_toggle), NULL);
+    return menuitem;
+}
+
 void
 rfm_null_function (void *user_data) {
     return ;
@@ -56,8 +94,8 @@ rfm_markup_stdout_f (void *user_data, void *stream, int childFD) {
         }
 
 	if (strchr(line, '-')){
-	    gchar buffer[256*5];
-	    memset (buffer, 0, 256*5);
+	    gchar buffer[256*10];
+	    memset (buffer, 0, 256*10);
 	    gint i;
 	    gint j;
 	    gboolean on=FALSE;
@@ -244,53 +282,6 @@ tooltip_unmap (GtkWidget *window, gpointer data){
 static void
 tooltip_map (GtkWidget *window, gpointer data){
     tooltip_is_mapped = TRUE;
-
-    gint x;
-    gint y;
-    gtk_window_get_position(GTK_WINDOW(window), &x, &y);
-    // This is a bug workaround for when gtk does something like:
-     if (x < 0 || y < 0) 
-    // on a race condition. But I rather like to do the placement 
-    // on a more rodent specific way, except for the flashing which
-    // is produced.
-#if 10
-	 DBG("tooltip_map(): this should not happen! A gtk bug?\n");
-#else
-     {
-        rfm_global_t *rfm_global_p = rfm_global();
-	TRACE( "tooltip is mapped...(%d, %d)\n", x, y);
-	TRACE("GTK tooltip placement bug workaround...\n");
-	gint px;
-	gint py;
-#if GTK_MAJOR_VERSION==3
-	gdk_device_get_position (rfm_global_p->pointer, NULL, &(px), &(py));
-#else 
-	gdk_display_get_pointer (gdk_display_get_default(), NULL, &(px), &(py), NULL);
-#endif
-	//gint offset;
-    
-	gint root_width;
-	gint root_height;
-	rfm_get_drawable_geometry (rfm_global_p->root_Xwindow, NULL, NULL, 
-	    &root_width, &root_height, NULL);	
-	GtkAllocation allocation;
-	gtk_widget_get_allocation(window, &allocation);
-	if (root_height - py - allocation.height > 20) {
-	    // underneath is OK.
-	    y = py + 20;
-	} else {
-	    y = root_height - allocation.height - 20;
-	}
-
-	if (root_width - px - allocation.width > 10){
-	    x = px + 10; // to the right is OK.
-	} else x = px - 10 - allocation.width;
-	TRACE( "GTK tooltip placement bug workaround...p=(%d,%d), r=(%d,%d), a=(%d,%d)\n",
-		px,py, root_width, root_height, 
-		x,y);
-	gtk_window_move(GTK_WINDOW(window), x, y);
-    }
-#endif                                           
 }
 
 static GdkPixbuf *
@@ -320,7 +311,72 @@ shadow_it(const GdkPixbuf *src_pixbuf){
     return shadowed;
 }
 
+void rfm_set_box_gradient(GtkWidget *wbox){
+    // only for gtk+3
+#if GTK_MAJOR_VERSION==3
+    
+    GtkStyleContext *style_context = gtk_widget_get_style_context (wbox);
+    gtk_style_context_add_class(style_context, GTK_STYLE_CLASS_TOOLTIP );
+     
+#if 0
+GtkGrid {\
+    background-image: -gtk-gradient (linear, left top, right top, from (#aaa), to (#000));\
+    color: rgb(0,0,0);\
+    background-color: rgb(255,255,255);\
+    text-shadow: 1px 1px 0 white, -1px -1px 0 white, 1px -1px 0 white, -1px 1px 0 white, 2px 2px 0 white, 1px 2px 0 white, 2px 1px 0 white;\
+}\
 
+#endif
+
+    GtkCssProvider *css_provider = gtk_css_provider_new();
+    GError *error=NULL;
+    gtk_css_provider_load_from_data (css_provider, 
+"\
+GtkEntry  {\
+    background-image: -gtk-gradient (linear, left top, right top, from (#888), to (#666));\
+    color: rgb(250,250,250);\
+    text-shadow: 0px 1px 0 black;\
+}\
+GtkBox,GtkEventBox,GtkFrame {\
+background-image: -gtk-gradient (linear, left top, right top, from (#aaa), to (#000));\
+color: rgb(255, 255, 255);\
+border-width: 0px;\
+border-radius: 0px;\
+border-color: transparent;\
+}\
+", 
+        -1, &error);
+    if (error){
+        fprintf(stderr, "gerror: %s\n", error->message);
+        g_error_free(error);
+    }
+    gtk_style_context_add_provider (style_context, GTK_STYLE_PROVIDER(css_provider),
+                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+#endif
+    return;
+}
+
+static void tooltip_placement_bug_workaround(GtkWidget *tooltip_window){
+#if GTK_MAJOR_VERSION==3
+#if GTK_MINOR_VERSION>=8
+    // gtk3.8 bug workaround (still in current 3.13):
+    static gint last_x = 0;
+    static gint last_y = 0;
+    
+    GdkScreen *screen = gtk_widget_get_screen (tooltip_window);
+    gint monitor_num = gdk_screen_get_monitor_at_point (screen,
+                                                 last_x,
+                                                 last_y);
+    GdkRectangle monitor;
+    gdk_screen_get_monitor_workarea (screen, monitor_num, &monitor);
+    last_x = monitor.width-1;
+    last_y = monitor.height-1;
+    gtk_window_move (GTK_WINDOW (tooltip_window), 
+            last_x,last_y);
+#endif
+#endif
+        return;
+}
 
 // If you just want the frame (as if to put in the properties dialog)
 // call this function with widget set to NULL. Frame is what this function
@@ -328,47 +384,45 @@ shadow_it(const GdkPixbuf *src_pixbuf){
 // If widget is not NULL, then the gtk tooltip  for the widget is set to the
 // window created by this function.
 GtkWidget *
-rfm_create_tooltip_window(GtkWidget *widget, const GdkPixbuf *pixbuf, const gchar *markup, const gchar *label_text){
-    GtkWidget *tooltip_window = NULL;
+rfm_create_tooltip_window(GtkWidget *widget, GtkWidget *tooltip_window, const GdkPixbuf *pixbuf, const gchar *markup, const gchar *label_text){
     if (widget) {
       gtk_widget_set_has_tooltip (widget, TRUE);
-      tooltip_window = gtk_window_new(GTK_WINDOW_POPUP);
-      NOOP(stderr, "New tooltip window now...\n");
-      g_signal_connect (G_OBJECT (tooltip_window), "map", G_CALLBACK (tooltip_map), NULL);
-      g_signal_connect (G_OBJECT (tooltip_window), "unmap", G_CALLBACK (tooltip_unmap), NULL);
+      if (!tooltip_window) {
+        tooltip_window = gtk_window_new(GTK_WINDOW_POPUP);
+        NOOP(stderr, "New tooltip window now...\n");
+        g_signal_connect (G_OBJECT (tooltip_window), "map", G_CALLBACK (tooltip_map), NULL);
+        g_signal_connect (G_OBJECT (tooltip_window), "unmap", G_CALLBACK (tooltip_unmap), NULL);
+        gtk_window_set_type_hint (GTK_WINDOW (tooltip_window), GDK_WINDOW_TYPE_HINT_TOOLTIP);
+        gtk_widget_set_app_paintable (tooltip_window, TRUE);
+        gtk_window_set_resizable (GTK_WINDOW (tooltip_window), FALSE);
+        gtk_widget_set_name (tooltip_window, "gtk-tooltip");  
 
-
-      gtk_window_set_type_hint (GTK_WINDOW (tooltip_window), GDK_WINDOW_TYPE_HINT_TOOLTIP);
-      gtk_widget_set_app_paintable (tooltip_window, TRUE);
-      gtk_window_set_resizable (GTK_WINDOW (tooltip_window), FALSE);
-      gtk_widget_set_name (tooltip_window, "gtk-tooltip");  
-#if GTK_MAJOR_VERSION==3
-    GdkRGBA black = {0.0, 0.0, 0.0, 1.0};
-    gtk_widget_override_background_color(tooltip_window, GTK_STATE_FLAG_NORMAL, &black);
-#else
-      GdkColor black = {0, 0, 0, 0}; 
-      gtk_widget_modify_bg(tooltip_window, GTK_STATE_NORMAL, &black);
+#if GTK_MAJOR_VERSION<3
+        GdkColor black = {0, 0, 0, 0}; 
+        gtk_widget_modify_bg(tooltip_window, GTK_STATE_NORMAL, &black);
 #endif
-    }
-    GtkWidget *top_frame = gtk_frame_new(NULL);
-    gtk_widget_show(top_frame);
-    if (widget) {
-	gtk_container_add (GTK_CONTAINER (tooltip_window), top_frame);
+
+      } else {
+        GtkWidget *old_content = 
+            gtk_bin_get_child(GTK_BIN(tooltip_window));
+        gtk_container_remove(GTK_CONTAINER(tooltip_window), old_content);
+      }
     }
 
     GtkWidget *vbox = rfm_vbox_new(FALSE, 2);
     gtk_widget_show(vbox);
-    gtk_container_add (GTK_CONTAINER (top_frame), vbox);
+
+    if (widget){
+        GtkWidget *top_frame = gtk_frame_new(NULL);
+        gtk_widget_show(top_frame);
+	gtk_container_add (GTK_CONTAINER (tooltip_window), top_frame);
+        gtk_container_add (GTK_CONTAINER (top_frame), vbox);
+    }
 
     GtkWidget *wbox  = gtk_event_box_new();
     if (widget) {
-	gint tip_bg = 0xf600;
-#if GTK_MAJOR_VERSION==3
-	gdouble tone = (gdouble)tip_bg / (gdouble)(0xffff);
-	GdkRGBA whitish = {tone, tone, tone, 1.0};
-    //    GdkRGBA whitish = {tone, tone, tone, 1.0};
-	gtk_widget_override_background_color(wbox, GTK_STATE_FLAG_NORMAL, &whitish);
-#else
+#if GTK_MAJOR_VERSION<3
+    gint tip_bg = 0xf600;
 	GdkColor whitish = {0, tip_bg, tip_bg, tip_bg};
 	if (widget) gtk_widget_modify_bg(wbox, GTK_STATE_NORMAL, &whitish);
 #endif
@@ -384,9 +438,17 @@ rfm_create_tooltip_window(GtkWidget *widget, const GdkPixbuf *pixbuf, const gcha
 	gchar *utf_text =  rfm_utf_string (label_text);
 	gchar *label_markup;
 	if (widget) {
+#if GTK_MAJOR_VERSION<3
 	    label_markup = g_strdup_printf("<span color=\"black\" font_family=\"monospace\" weight=\"bold\">%s</span>",utf_text); 
+#else
+	    label_markup = g_strdup_printf("<span font_family=\"monospace\" weight=\"bold\">%s</span>",utf_text); 
+#endif
 	} else {
+#if GTK_MAJOR_VERSION<3
 	    label_markup = g_strdup_printf("<span color=\"black\" font_family=\"monospace\" size=\"larger\" weight=\"bold\">%s</span>\n",utf_text); 
+#else
+	    label_markup = g_strdup_printf("<span font_family=\"monospace\" size=\"larger\" weight=\"bold\">%s</span>\n",utf_text); 
+#endif
 	}
 	gtk_label_set_markup(GTK_LABEL(label), label_markup);
 	g_free(utf_text);
@@ -421,7 +483,13 @@ rfm_create_tooltip_window(GtkWidget *widget, const GdkPixbuf *pixbuf, const gcha
 	//gtk_widget_set_size_request (label, 60, -1);
 	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 	if (widget) {
+//	    gchar *small = g_strdup_printf("<span color=\"white\" size=\"smaller\">%s</span>", markup);
+#if GTK_MAJOR_VERSION<3
 	    gchar *small = g_strdup_printf("<span color=\"black\" size=\"smaller\">%s</span>", markup);
+	    //gchar *small = g_strdup_printf("<span background=\"white\" color=\"black\" size=\"smaller\">%s</span>", markup);
+#else
+	    gchar *small = g_strdup_printf("<span size=\"smaller\">%s</span>", markup);
+#endif
 	    gtk_label_set_markup(GTK_LABEL(label), small);
 	    g_free(small);
 
@@ -448,26 +516,12 @@ rfm_create_tooltip_window(GtkWidget *widget, const GdkPixbuf *pixbuf, const gcha
 
 	gtk_widget_set_tooltip_window (widget, GTK_WINDOW(tooltip_window));
 	gtk_widget_realize(tooltip_window);
-
-#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 8
-        // gtk3.8 bug workaround (still in current 3.13):
-        static gint last_x = 0;
-        static gint last_y = 0;
-        
-        GdkScreen *screen = gtk_widget_get_screen (tooltip_window);
-        gint monitor_num = gdk_screen_get_monitor_at_point (screen,
-                                                     last_x,
-                                                     last_y);
-        GdkRectangle monitor;
-        gdk_screen_get_monitor_workarea (screen, monitor_num, &monitor);
-        last_x = monitor.width-1;
-        last_y = monitor.height-1;
-        gtk_window_move (GTK_WINDOW (tooltip_window), 
-                last_x,last_y);
-#endif
+        rfm_set_box_gradient(wbox);
+        tooltip_placement_bug_workaround(tooltip_window);
 	return tooltip_window;
     }
-    return top_frame;
+    rfm_set_box_gradient(wbox);
+    return vbox;
 }
 
 static void
@@ -673,17 +727,21 @@ little_button_press (GtkWidget *button, GdkEventButton  *event, gpointer callbac
 }
 
     
-static GtkWidget *tooltip_window = NULL;
+static GtkWidget *tt_window = NULL;
 
 void
 rfm_reset_tooltip(GtkWidget * widget){
     NOOP(stderr, "rfm_reset_tooltip\n"); 
-    if (tooltip_window) {
-	g_object_set_data(G_OBJECT(tooltip_window), "tooltip_target", NULL);
+    if (tt_window) {
+	g_object_set_data(G_OBJECT(tt_window), "tooltip_target", NULL);
 	NOOP(stderr, "rfm_reset_tooltip OK\n"); 
     }
 }
 
+// FIXME: on view destroy, all tooltip texts associated to widgets (button mainly)
+//        should be freed and removed from hashtable. (small leak here)
+static GHashTable *tooltip_text_hash = NULL;
+
 static gboolean
 widget_tooltip_function(
         GtkWidget * widget, 
@@ -693,11 +751,10 @@ widget_tooltip_function(
         GtkTooltip * tooltip,
         gpointer user_data
 ) {
-    if (tooltip_window) {
+    if (tt_window) {
 	GtkWidget *tooltip_target = 
-	    g_object_get_data(G_OBJECT(tooltip_window), "tooltip_target");
+	    g_object_get_data(G_OBJECT(tt_window), "tooltip_target");
 	if (tooltip_target == widget) return TRUE;
-	gtk_widget_destroy(tooltip_window);
     }
 
     GdkPixbuf *tooltip_pixbuf = g_object_get_data(G_OBJECT(widget), "tooltip_pixbuf");
@@ -712,8 +769,9 @@ widget_tooltip_function(
 	    tooltip_text = strchr(tooltip_text, '\n') + 1;
 	}
     }
-    tooltip_window = rfm_create_tooltip_window(widget, tooltip_pixbuf, tooltip_text, label_text);
-    g_object_set_data(G_OBJECT(tooltip_window), "tooltip_target", widget);
+    tt_window = rfm_create_tooltip_window(widget, tt_window, tooltip_pixbuf, tooltip_text, label_text);
+
+    g_object_set_data(G_OBJECT(tt_window), "tooltip_target", widget);
 
     g_free(label_text);
     return TRUE;
@@ -722,8 +780,14 @@ widget_tooltip_function(
 static void
 destroy_widget(GtkWidget *button, void *data){
     GdkPixbuf *tooltip_pixbuf = g_object_get_data(G_OBJECT(button), "tooltip_pixbuf");
-    gchar *tooltip_text = g_object_get_data(G_OBJECT(button), "tooltip_text");
-    g_free(tooltip_text);
+    gchar *tooltip_text = g_hash_table_lookup(tooltip_text_hash, button);
+//        g_object_get_data(G_OBJECT(button), "tooltip_text");
+    if (tooltip_text) {
+        // The free is done by removing item from hash table:
+        // g_free(tooltip_text);
+        g_hash_table_remove(tooltip_text_hash, button);
+    }
+
     if (tooltip_pixbuf) g_object_unref(tooltip_pixbuf);
     g_object_set_data(G_OBJECT(button), "tooltip_text", NULL);
     g_object_set_data(G_OBJECT(button), "tooltip_pixbuf", NULL);
@@ -735,7 +799,12 @@ custom_tooltip_f(gpointer data){
     GtkWidget *widget = arg[0];
     GdkPixbuf *pixbuf = arg[1];
     const gchar *text = arg[2];
-    g_object_set_data(G_OBJECT(widget), "tooltip_text", g_strdup(text));
+    if (!tooltip_text_hash) {
+        tooltip_text_hash = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
+    }
+    gchar *t = g_strdup(text);
+    g_object_set_data(G_OBJECT(widget), "tooltip_text", t);
+    g_hash_table_replace(tooltip_text_hash, widget, t);
     g_object_set_data(G_OBJECT(widget), "tooltip_pixbuf", pixbuf);
     if (pixbuf) g_object_ref(pixbuf);
 
@@ -743,7 +812,7 @@ custom_tooltip_f(gpointer data){
     g_signal_connect (G_OBJECT (widget), "destroy",
 	    G_CALLBACK (destroy_widget), NULL);
     g_signal_connect (G_OBJECT (widget), "query-tooltip",
-	    G_CALLBACK (widget_tooltip_function), NULL);
+	    G_CALLBACK (widget_tooltip_function), widget);
     return NULL;
 }
 
@@ -788,7 +857,7 @@ rfm_cursor_wait (gpointer data) {
     GtkWidget * widget = data;
     static GdkCursor *cursor = NULL;
     if(!widget) return NULL;
-    if(!cursor) cursor = gdk_cursor_new (GDK_WATCH);
+    if(!cursor) cursor = gdk_cursor_new_for_display (gdk_display_get_default(),GDK_WATCH);
     gdk_window_set_cursor (gtk_widget_get_window(widget), cursor);
     return NULL;
 }
@@ -867,6 +936,7 @@ rfm_create_diagnostics_window (widgets_t * widgets_p) {
     //// THis instead:
     dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_keep_above (GTK_WINDOW(dialog), TRUE);
+    gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 
 
     gtk_window_stick (GTK_WINDOW(dialog));
@@ -898,18 +968,27 @@ rfm_create_diagnostics_window (widgets_t * widgets_p) {
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW ((widgets_p->diagnostics)), GTK_WRAP_WORD);
     gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW ((widgets_p->diagnostics)), FALSE);
 
+#if 0
+    oobbsoolete. just check if needed for desktop console with gtk2 (probably not)
+#if GTK_MAJOR_VERSION==2
+    	gint size = 10; //8 This only for desktop....
+        PangoFontDescription *font_desc = pango_font_description_new ();
+        pango_font_description_set_family (font_desc, "monospace");
+        pango_font_description_set_size (font_desc, size * PANGO_SCALE);
+	gtk_widget_modify_font ((widgets_p->diagnostics), font_desc);
+	pango_font_description_free (font_desc); 
+#endif
+
+#if GTK_MAJOR_VERSION==3
+        // GTK_MINOR_VERSION>=15 is taken care of with style below.
     	gint size = 10; //8 This only works for desktop....
         PangoFontDescription *font_desc = pango_font_description_new ();
         pango_font_description_set_family (font_desc, "monospace");
         pango_font_description_set_size (font_desc, size * PANGO_SCALE);
-#if GTK_MAJOR_VERSION==2
-	gtk_widget_modify_font ((widgets_p->diagnostics), font_desc);
-#else
-	gtk_widget_override_font ((widgets_p->diagnostics), font_desc);
-#endif
+        gtk_widget_override_font ((widgets_p->diagnostics), font_desc);
 	pango_font_description_free (font_desc); 
-	//g_object_set_data(G_OBJECT((widgets_p->diagnostics)), 
-	//	"font_desc", font_desc);
+#endif
+#endif
 
     GtkWidget *vbox = rfm_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
@@ -942,6 +1021,7 @@ rfm_create_diagnostics_window (widgets_t * widgets_p) {
     widgets_p->button_space = rfm_hbox_new (FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), widgets_p->button_space, FALSE, FALSE, 0);
     gtk_widget_realize (dialog);
+
     if(!getenv ("RFM_ENABLE_DESKTOP_DIAGNOSTICS") ||
 	  strlen (getenv ("RFM_ENABLE_DESKTOP_DIAGNOSTICS"))==0)
     {
@@ -1867,23 +1947,39 @@ set_font_family (GtkWidget * widget, const gchar *in_family, gboolean fixed) {
 	    g_object_set_data(G_OBJECT(widget), "font-family", family);
 	}
 	fontsize = newsize;
+	g_object_set_data(G_OBJECT(widget), 
+		"fontsize", GINT_TO_POINTER(fontsize));
+
+#if GTK_MAJOR_VERSION==2
 	gchar *font_id = g_strdup_printf("%s_font_desc", family);
-        //PangoFontDescription *old_desc = 
-	  //  g_object_get_data(G_OBJECT(widget), font_id);
         PangoFontDescription *font_desc = pango_font_description_new ();
         pango_font_description_set_family (font_desc, family);
         pango_font_description_set_size (font_desc, fontsize * PANGO_SCALE);
-#if GTK_MAJOR_VERSION==2
 	gtk_widget_modify_font (widget, font_desc);
-#else
-	gtk_widget_override_font (widget, font_desc);
-#endif
-	g_object_set_data(G_OBJECT(widget), 
-		"fontsize", GINT_TO_POINTER(fontsize));
-	//g_object_set_data(G_OBJECT(widget), font_id, font_desc);
 	g_free(font_id);
 	pango_font_description_free (font_desc); 
-	//if (old_desc) pango_font_description_free (old_desc); 
+#else
+        GtkStyleContext *style_context = gtk_widget_get_style_context (widget);
+        gtk_style_context_add_class(style_context, GTK_STYLE_CLASS_VIEW );
+        GtkCssProvider *css_provider = gtk_css_provider_new();
+        GError *error=NULL;
+        gchar *data = g_strdup_printf("* {\
+font-family: %s;\
+font-size: %dpx;\
+}", family, fontsize);
+        gtk_css_provider_load_from_data (css_provider, data, -1, &error);
+        g_free(data);
+        if (error){
+            fprintf(stderr, "gerror: %s\n", error->message);
+            g_error_free(error);
+        }
+        gtk_style_context_add_provider (style_context, GTK_STYLE_PROVIDER(css_provider),
+                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+         
+//	gtk_widget_override_font (widget, font_desc);
+#endif
+
+
     }
 }
 
@@ -1900,11 +1996,6 @@ rfm_status (widgets_t * widgets_p, const gchar * id, ...) {
     GdkPixbuf *icon;
     //gint icon_width = 0;
 
-    if (!widgets_p) {
-        return;
-    }
-//    if(!widgets_p) return;
-
     if(!widgets_p->status || !GTK_IS_TEXT_VIEW (widgets_p->status)) {
 #ifdef DEBUG_NOOP
         /*DBG("rfm_status: status==NULL\n"); */
@@ -2395,7 +2486,6 @@ rfm_toggle_button (const char *icon_id, const char *text) {
 }
 
 
-#if GTK_MAJOR_VERSION>=3
 GtkWidget *
 rfm_pathbar_button (const char *icon_id, const char *text) {
     //GtkWidget *pb_button = gtk_toggle_button_new ();
@@ -2406,10 +2496,14 @@ rfm_pathbar_button (const char *icon_id, const char *text) {
             "relief", GTK_RELIEF_NONE, 
             NULL);
     g_object_set_data(G_OBJECT(pb_button), "name", text?g_strdup(text):NULL); 
-    gchar *g = text?g_markup_escape_text(text, -1):NULL;
-    gchar *markup = g?g_strdup_printf("<span size=\"x-small\">%s</span>", g):NULL;
-    g_free(g);
-
+    gchar *markup = NULL;
+    if (text) {
+        gchar *v = rfm_utf_string(text);
+        gchar *g = g_markup_escape_text(v, -1);
+        g_free(v);
+        markup = g_strdup_printf("<span size=\"x-small\">%s</span>", g);
+        g_free(g);
+    }
     set_bin_contents(pb_button, icon_id, markup, 12);
     g_free(markup);
     return pb_button;
@@ -2432,8 +2526,17 @@ static void toggle_pathbar(GtkWidget *pathbar, const gchar *path){
     for (;children && children->data; children=children->prev){
         gchar *name = g_object_get_data(G_OBJECT(children->data), "name");
         if (strcmp(name, "RFM_ROOT")==0) {
+#if GTK_MAJOR_VERSION>=3
             gtk_widget_get_preferred_size(GTK_WIDGET(children->data), 
                     &minimum, NULL);
+#else
+    {
+     GtkAllocation a;
+     gtk_widget_get_allocation(GTK_WIDGET(children->data), &a);
+     minimum.width = a.width;
+    }
+
+#endif
             width -= minimum.width;
             continue;
         }
@@ -2459,10 +2562,16 @@ static void toggle_pathbar(GtkWidget *pathbar, const gchar *path){
 
     // Show active button
     gtk_widget_show(GTK_WIDGET(active->data));
+#if GTK_MAJOR_VERSION>=3
     gtk_widget_get_preferred_size(GTK_WIDGET(active->data), &minimum, NULL);
+#else
+    {
+     GtkAllocation a;
+     gtk_widget_get_allocation(GTK_WIDGET(active->data), &a);
+     minimum.width = a.width;
+    }
+#endif
     width -= minimum.width;
-    //gchar *name = g_object_get_data(G_OBJECT(active->data), "name");
-    //DBG("F---->  %s: minimum width = %d (%d)\n", name, minimum.width, width);
  
     // Work backwards from active button we show buttons that will fit.
     children = active->prev;
@@ -2473,8 +2582,6 @@ static void toggle_pathbar(GtkWidget *pathbar, const gchar *path){
         width -= allocation.width;
         if (width < 0) break;
         gtk_widget_show(GTK_WIDGET(children->data));
-        //gchar *name = g_object_get_data(G_OBJECT(children->data), "name");
-        //DBG("---->  %s: allocation width = %d (%d)\n", name, allocation.width, width);
     }
 
     // Now we work forwards, showing buttons that fit.
@@ -2495,7 +2602,9 @@ static void toggle_pathbar(GtkWidget *pathbar, const gchar *path){
         if (strcmp(name, "RFM_ROOT")==0) continue;
         if (!path) {
             // no path means none is differentiated.
-            gchar *g = g_markup_escape_text(name, -1);
+            gchar *v = rfm_utf_string(name);
+            gchar *g = g_markup_escape_text(v, -1);
+            g_free(v);
             gchar *markup = g_strdup_printf("<span size=\"x-small\" color=\"blue\" bgcolor=\"#dcdad5\">%s</span>", g);
             rfm_set_bin_markup(GTK_WIDGET(children->data), markup);
             g_free(g);
@@ -2510,14 +2619,18 @@ static void toggle_pathbar(GtkWidget *pathbar, const gchar *path){
         }
         if (!strlen(pb_path)) pb_path=G_DIR_SEPARATOR_S;//?
         if (strcmp(pb_path, path)==0) {
-            gchar *g = g_markup_escape_text(name, -1);
+            gchar *v = rfm_utf_string(name);
+            gchar *g = g_markup_escape_text(v, -1);
+            g_free(v);
             gchar *markup = g_strdup_printf("<span size=\"x-small\" color=\"red\"bgcolor=\"#dcdad5\">%s</span>", g);
             rfm_set_bin_markup(GTK_WIDGET(children->data), markup);
             g_free(g);
             g_free(markup);
         }
         else {
-            gchar *g = g_markup_escape_text(name, -1);
+            gchar *v = rfm_utf_string(name);
+            gchar *g = g_markup_escape_text(v, -1);
+            g_free(v);
             gchar *markup = g_strdup_printf("<span size=\"x-small\" color=\"blue\"bgcolor=\"#dcdad5\">%s</span>", g);
             rfm_set_bin_markup(GTK_WIDGET(children->data), markup);
             g_free(g);
@@ -2586,10 +2699,15 @@ static void *rfm_update_pathbar_f(void *data){
 
     // Add new tail
     gpointer callback =g_object_get_data(G_OBJECT(pathbar), "callback");
-    if (strcmp(path, "RFM_MODULE")) for (;paths && paths[i]; i++){
+    if (strcmp(path, "RFM_MODULE")) for (;paths[i]; i++){
         GtkWidget *pb_button = rfm_pathbar_button(NULL, 
                 strlen(paths[i])?paths[i]:G_DIR_SEPARATOR_S);
+#if GTK_MAJOR_VERSION>=3
         gtk_container_add(GTK_CONTAINER(pathbar), pb_button);
+
+#else
+        gtk_box_pack_start(GTK_BOX(pathbar), pb_button, FALSE, FALSE, 1);
+#endif
         gchar *g = (pb_path!=NULL)?
             g_strdup_printf("%s%s%s",pb_path, 
                     strcmp(pb_path,G_DIR_SEPARATOR_S)? 
@@ -2616,7 +2734,6 @@ void rfm_update_pathbar(GtkWidget *pathbar, const gchar *path){
     rfm_context_function(rfm_update_pathbar_f, arg);
 }
 
-#endif
 
 static void *
 dialog_run_response_f(void *data){
