@@ -37,6 +37,19 @@ typedef struct mime_winner_thread_t {
 // This cleans up the thumbnail cache when reload is pressed in order
 // that thumbnails be unconditionally regenerated.
 
+gint 
+rfm_get_preview_image_size(void){
+    gchar *s_size = getenv("RFM_PREVIEW_IMAGE_SIZE");
+    if (!s_size || !strlen(s_size)) return PREVIEW_IMAGE_SIZE;
+    errno = 0;
+    long size = strtol(s_size, NULL, 10);
+    if (errno){
+        fprintf(stderr, "strtol(%s): %s\n", s_size, strerror(errno));
+        return PREVIEW_IMAGE_SIZE;
+    }
+    if (size < 24 || size > 1000) return PREVIEW_IMAGE_SIZE;
+    return (gint)size;
+}
 
 void *
 rfm_cleanup_thumbnails(void *data){
@@ -84,7 +97,7 @@ rfm_cleanup_thumbnails(void *data){
             if(strcmp (d->d_name, ".") == 0) continue;
             if(strcmp (d->d_name, "..") == 0) continue;
             gchar *fullpath = g_build_filename (path, d->d_name, NULL);
-            rfm_rm_from_pixbuf_hash(fullpath, PREVIEW_IMAGE_SIZE);
+            rfm_rm_from_pixbuf_hash(fullpath, rfm_get_preview_image_size());
             rfm_rm_from_pixbuf_hash(fullpath, TINY_ICON_SIZE);
             rfm_rm_from_pixbuf_hash(fullpath, SMALL_ICON_SIZE);
             rfm_rm_from_pixbuf_hash(fullpath, MEDIUM_ICON_SIZE);
@@ -508,16 +521,15 @@ rfm_create_preview (const gchar * file, gint size) {
         if (stat (thumbnail_path, &thumbnail_st) < 0)stat_ok = FALSE;
 	    
 	// This already succeded in the path_is_image() call.
-        stat (file, &file_st);
+        if (stat (file, &file_st) != 0) goto done;
         if(stat_ok && thumbnail_st.st_mtime > file_st.st_mtime) {
             NOOP ("PREVIEW thread: retrieving thumbnail for %s (%s)\n", file, thumbnail_path);
 	    if (g_file_test(thumbnail_path, G_FILE_TEST_EXISTS)){
 		pixbuf = rfm_pixbuf_new_from_file(thumbnail_path, -1, -1); // check- OK contexted.
 	    }
             if(pixbuf && GDK_IS_PIXBUF(pixbuf) ) {
-                g_free (thumbnail_path);
 		g_object_ref(pixbuf);
-                return pixbuf;
+                goto done;
             }
         }
 	if (g_file_test(thumbnail_path, G_FILE_TEST_EXISTS) && unlink(thumbnail_path)<0){
@@ -533,6 +545,7 @@ rfm_create_preview (const gchar * file, gint size) {
     } else {
 	DBG("could not create pixbuf from: %s at size %dx%d\n", file, size, size);
     }
+done:
     g_free (thumbnail_path);
 	    
     return pixbuf;
