@@ -187,7 +187,6 @@ main_context_function_f(gpointer data){
     GMutex *mutex = arg[2];
     GCond *signal = arg[3];
     void **result_p = arg[4];
-    g_free(arg);
     void *result = (*function)(function_data);
     g_mutex_lock(mutex);
     *result_p = result;
@@ -198,8 +197,7 @@ main_context_function_f(gpointer data){
 
 void *
 rfm_context_function(void * (*function)(gpointer), void * function_data){
-    void **arg = (void **)malloc(5*sizeof(void*));
-    if (!arg) g_error("malloc: %s\n", strerror(errno));
+    void *arg[5];
     GMutex *mutex = NULL;
     rfm_mutex_init(mutex);
     GCond *signal;
@@ -728,11 +726,31 @@ rfm_esc_string (const gchar * string) {
 const gchar *
 rfm_chop_excess (gchar * b) {
     // chop excess length...
-    int len = strlen (b);
-    if(len > MAX_PATH_LABEL) {
-        b[MAX_PATH_START_LABEL - 1] = '~';
-        strcpy (b + MAX_PATH_START_LABEL, b + (len - (MAX_PATH_LABEL - MAX_PATH_START_LABEL)));
+
+    const gchar *home = g_get_home_dir();
+    gchar *bb;
+    if (strncmp(home, b, strlen(home))==0){
+        if (strlen(home) == strlen(b)) return b;
+        bb = g_strconcat("~/", b + strlen(home)+1, NULL);
+    } else {
+        bb = g_strdup(b);
     }
+    
+    int len = strlen (bb);
+
+    if(len < MAX_PATH_LABEL) {
+        strcpy(b, bb);
+        g_free(bb);
+        return (b);
+    }
+        
+    bb[MAX_PATH_START_LABEL - 3] = 0;
+
+    gchar *g = g_strconcat(bb, "...", b + (len - MAX_PATH_LABEL + MAX_PATH_START_LABEL), NULL);
+    strcpy (b, g);
+    g_free(bb);
+    g_free(g);
+
     return b;
 }
 
@@ -819,10 +837,8 @@ rfm_group_string (struct stat *st) {
     gchar *group_string;
     if((g =  getgrgid(st->st_gid)) != NULL)
             group_string = g_strdup(g->gr_name);
-        else if((gint)st->st_gid < 0)
-            group_string = g_strdup("");
-        else
-            group_string = g_strdup_printf("%d", (gint)st->st_gid);
+    else
+        group_string = g_strdup_printf("%d", (gint)st->st_gid);
     pthread_mutex_unlock(&group_string_mutex);
     return group_string;
 }
