@@ -26,6 +26,8 @@
 #include "rfm_modules.h"
 
 #define CONTROL_MODE (event->state & GDK_CONTROL_MASK)
+#define SHIFT_MODE (event->state & GDK_SHIFT_MASK)
+#define ALT_MODE ((event->state & GDK_MOD1_MASK)|| (event->state & GDK_MOD5_MASK))
 #define NO_CLICK 0
 #define LABEL_CLICK 1
 #define ICON_CLICK 2
@@ -67,7 +69,7 @@ double_click(view_t *view_p, const population_t *population_p, GdkEventButton *
     }
 
     
-    if((CONTROL_MODE || event->state & GDK_SHIFT_MASK) &&
+    if((CONTROL_MODE || SHIFT_MODE)  &&
 	    view_p->mouse_event.single_click_mode_state != LABEL_CLICK) {
 	// Update the number of selected items in status line:
 	rfm_update_status_line (view_p);
@@ -99,13 +101,11 @@ double_click(view_t *view_p, const population_t *population_p, GdkEventButton *
                && !IS_DUMMY_TYPE (population_p->en->type)
                && population_p->en->path) {
                 gint caso = RENAME_ACTIVATE;
-                if(event->state & GDK_CONTROL_MASK)
+                if(CONTROL_MODE)
                     caso = DUPLICATE_ACTIVATE;
-                if(event->state & GDK_MOD1_MASK)
+                if(ALT_MODE)
                     caso = SYMLINK_ACTIVATE;
-                if(event->state & GDK_MOD5_MASK)
-                    caso = SYMLINK_ACTIVATE;
-                if(event->state & GDK_SHIFT_MASK && event->state & GDK_CONTROL_MASK)
+                if(SHIFT_MODE && CONTROL_MODE)
                     caso = SYMLINK_ACTIVATE;
                 //view_p->mouse_event.selected_p = population_p;
 		gboolean modular = view_p->en == NULL || 
@@ -135,10 +135,11 @@ rodent_signal_on_button_press (GtkWidget * widget,
 	    getenv("RFM_DOUBLE_CLICK_NAVIGATION"),
 	    getenv("RFM_DOUBLE_CLICK_NAVIGATION")?
 	    strlen(getenv("RFM_DOUBLE_CLICK_NAVIGATION")):0);
-    NOOP ("rodent_mouse:  button_press 0x%x(%d) x=%lf y=%lf\n", 
+    NOOP (stderr, "rodent_mouse:  button_press 0x%x(%d) x=%lf y=%lf\n", 
 	    GPOINTER_TO_INT(event), event->button, event->x, event->y);
 
     if (event->button == 2) return TRUE;
+    if (ALT_MODE) return TRUE;
     view_t *view_p = (view_t *) data;
     rfm_set_widget(&(view_p->widgets),"widgets_p");
     /* cancel any pending tip : */
@@ -175,6 +176,10 @@ rodent_signal_on_button_press (GtkWidget * widget,
     rfm_global_t *rfm_global_p = rfm_global();
     if (population_p && population_p->en && population_p->en->path){
 	rfm_cursor_wait(rfm_global_p->window); 
+	if (ALT_MODE) rfm_cursor_reset(rfm_global_p->window); 
+	if (CONTROL_MODE) rfm_cursor_reset(rfm_global_p->window); 
+	if (SHIFT_MODE) rfm_cursor_reset(rfm_global_p->window); 
+
 	if (g_path_is_absolute(population_p->en->path)){
 	    gchar *base = g_path_get_basename(population_p->en->path);
 	    rfm_status(&(view_p->widgets), population_p->icon_id, base, NULL);
@@ -217,7 +222,7 @@ rodent_signal_on_button_press (GtkWidget * widget,
         view_p->mouse_event.selected_p = NULL;
         NOOP (stderr, "rodent_mouse:  starting rubberband selection at %f %f\n",
 		event->x, event->y);
-        if(!(event->state & GDK_CONTROL_MASK)) {
+        if(!CONTROL_MODE) {
             unselect_all_pixbuf (view_p);
             unsaturate_icon (view_p);
             view_p->mouse_event.selected_p = NULL;
@@ -242,7 +247,7 @@ rodent_signal_on_button_press (GtkWidget * widget,
 
     view_p->mouse_event.dragstate = FALSE;
 
-    if(event->state & GDK_SHIFT_MASK && !label_click) {
+    if(SHIFT_MODE && !label_click) {
 	// Add items to the selection list.
 	if (population_p && view_p->population_pp){
 	    NOOP("rodent_mouse:  adding 0x%x to selection list\n",
@@ -261,7 +266,7 @@ rodent_signal_on_button_press (GtkWidget * widget,
 		     || p==population_p);
 		if (to_c && pp!=from) to=pp;
 	    }
-	    if (*from != population_p && *to  != population_p) to=NULL;
+	    if (from && *from != population_p && to && *to  != population_p) to=NULL;
 	    if (!to) to=from;
 	    rodent_unselect_all_pixbuf (view_p);
 	    for (pp=from; pp && *pp; pp++){
@@ -294,7 +299,8 @@ rodent_signal_on_button_press (GtkWidget * widget,
     view_p->mouse_event.rubberbanding = FALSE;
     view_p->mouse_event.old_X = view_p->mouse_event.old_Y = view_p->mouse_event.boxX = view_p->mouse_event.boxY = -1;
     int x_spacing;
-    x_spacing = (CELLWIDTH(view_p) - population_p->icon_size) / 2;
+    //x_spacing = (CELLWIDTH(view_p) - population_p->icon_size) / 2;
+    x_spacing = (CELLWIDTH(view_p) - ICON_SIZE(view_p)) / 2;
     if(x_spacing < 0) {
         x_spacing = 0;
     }
@@ -333,12 +339,15 @@ rodent_signal_on_button_press (GtkWidget * widget,
 
 gboolean
 rodent_signal_on_button_release (GtkWidget * widget, GdkEventButton * event, gpointer data) {
-    NOOP ("rodent_mouse:  button_release 0x%x(%d) x=%lf y=%lf\n", 
+    NOOP (stderr,"rodent_mouse:  button_release 0x%x(%d) x=%lf y=%lf\n", 
 	    GPOINTER_TO_INT(event), event->button, event->x, event->y);
 
     view_t *view_p = (view_t *) data;
 
-    if (view_p->mouse_event.rubberbanding && event->button != 1) return TRUE;
+    if (view_p->mouse_event.rubberbanding && event->button != 1){
+        NOOP( "condition 1 return\n");
+        return TRUE;
+    }
     const population_t *population_p;
     if(event->x < 0){
         event->x = 0;
@@ -350,10 +359,13 @@ rodent_signal_on_button_release (GtkWidget * widget, GdkEventButton * event, gpo
     view_p->mouse_event.mouseX = -1;
     view_p->mouse_event.mouseY = -1;
 
-    if (view_p->mouse_event.doing_drag_p) return TRUE;
+    if (view_p->mouse_event.doing_drag_p){
+        NOOP( "condition 2 return\n");
+        return TRUE;
+    }
     rfm_global_t *rfm_global_p = rfm_global();
     if(!rfm_population_try_read_lock (view_p, "rodent_signal_on_button_release")) {
-	NOOP("rodent_mouse:  button_release 0x%x(%d) done\n", 
+	NOOP( "rodent_mouse: no read lock. button_release 0x%x(%d) done\n", 
 	    GPOINTER_TO_INT(event), event->button);
 	rfm_cursor_reset(rfm_global_p->window);
         return TRUE;
@@ -365,14 +377,14 @@ rodent_signal_on_button_release (GtkWidget * widget, GdkEventButton * event, gpo
         population_p = rodent_find_in_labels (view_p, event->x, event->y);
     }
 
-    if(population_p && CONTROL_MODE) {
+    if(population_p && (CONTROL_MODE || ALT_MODE)) {
         if(!(population_p->flags  & POPULATION_SELECTED)) {
-	    NOOP("rodent_mouse: selecting single population_p 0x%x\n",
+	    NOOP( "rodent_mouse: selecting single population_p 0x%x\n",
 		GPOINTER_TO_INT(population_p));
 	    rfm_select_pixbuf (view_p, population_p);
 	    rfm_expose_item (view_p, population_p);
         } else {
-	    NOOP("rodent_mouse: unselecting single population_p 0x%x\n",
+	    NOOP( "rodent_mouse: unselecting single population_p 0x%x\n",
 		GPOINTER_TO_INT(population_p));
             rfm_unselect_pixbuf (view_p, population_p);
 	    rfm_expose_item(view_p, population_p);
@@ -381,14 +393,17 @@ rodent_signal_on_button_release (GtkWidget * widget, GdkEventButton * event, gpo
     }
 
 
-    if(SINGLE_CLICK_MODE && (event->button == 1)) {
+    if(SINGLE_CLICK_MODE && (event->button == 1) && !ALT_MODE) {
+        NOOP( "release: double click now...\n");
 	double_click(view_p, population_p, event);
     } else if(event->button == 1) {
-	rfm_cursor_reset(rfm_global_p->window);	
+        NOOP( "rfm_cursor_reset...\n");
     }
+    rfm_cursor_reset(rfm_global_p->window);	
 
     view_p->mouse_event.doing_drag_p = NULL;
     if (view_p->mouse_event.rubberbanding){
+        NOOP( "release: view_p->mouse_event.rubberbanding...\n");
 	view_p->mouse_event.rubberbanding = FALSE;
 	// rubber band function will do the final expose.
 	rubber_band (view_p, event->x, event->y, TRUE);
@@ -402,7 +417,10 @@ rodent_signal_on_button_release (GtkWidget * widget, GdkEventButton * event, gpo
 //////
     rfm_population_read_unlock (view_p, "rodent_signal_on_button_release");
 
-    NOOP("rodent_mouse:  button_release 0x%x(%d) done\n", 
+    if (population_p && ALT_MODE){
+	    rodent_activate_tip(view_p, (population_t *)population_p, TRUE);
+    }
+    NOOP( "rodent_mouse:  button_release 0x%x(%d) done\n", 
 	GPOINTER_TO_INT(event), event->button);
     return TRUE;
 }
@@ -410,7 +428,7 @@ rodent_signal_on_button_release (GtkWidget * widget, GdkEventButton * event, gpo
 gboolean
 rodent_signal_on_motion (GtkWidget * widget, GdkEventMotion * event, gpointer data) {
 
-    NOOP("rodent_mouse:  on_motion\n");
+    NOOP(stderr, "rodent_mouse:  on_motion\n");
     view_t *view_p = (view_t *) data;
     //widgets_t *widgets_p = &(view_p->widgets);
 
@@ -430,6 +448,7 @@ rodent_signal_on_motion (GtkWidget * widget, GdkEventMotion * event, gpointer da
             view_p->mouse_event.single_click_mode_state = NO_CLICK;
 	}
 	XUngrabServer(rfm_global_p->Xdisplay);
+	NOOP( "mouse rubberbanding\n");
         return TRUE;
     }  
 
@@ -437,7 +456,7 @@ rodent_signal_on_motion (GtkWidget * widget, GdkEventMotion * event, gpointer da
     if(!rfm_population_try_read_lock (view_p, "rodent_signal_on_motion")) {
         return TRUE;
     }
-    NOOP("rodent_mouse:  on_motion, population read-lock obtained...\n");
+    NOOP( "rodent_mouse:  on_motion, population read-lock obtained...\n");
 
 
     // If icon has been dragged more than a set distance, mark population
@@ -461,6 +480,9 @@ rodent_signal_on_motion (GtkWidget * widget, GdkEventMotion * event, gpointer da
 	if ((motionX * motionX) + (motionY * motionY) >= 4.0){
 	    NOOP("rodent_mouse:  on motion, turning drag on... 0x%x\n",
 		GPOINTER_TO_INT(view_p->mouse_event.doing_drag_p));
+            if (view_p->mouse_event.drag_action == GDK_ACTION_DEFAULT) {
+                view_p->mouse_event.drag_action = GDK_ACTION_COPY;
+            }
 	    enter_drag_state (view_p);
 	}
     }
@@ -473,7 +495,7 @@ rodent_signal_on_motion (GtkWidget * widget, GdkEventMotion * event, gpointer da
     g_mutex_unlock(view_p->mutexes.status_mutex);
 
     if(population_p) {
-    NOOP(stderr, "rodent_signal_on_motion: tip_function TRUE ... \n");
+        NOOP( "rodent_signal_on_motion: tip_function TRUE ... \n");
 	unsaturate_label (view_p);
 	saturate_icon (view_p, population_p);
 
@@ -484,13 +506,15 @@ rodent_signal_on_motion (GtkWidget * widget, GdkEventMotion * event, gpointer da
 	rfm_population_read_unlock (view_p, "rodent_signal_on_motion");
 	return TRUE;
     } 
+    NOOP( "rodent_signal_on_motion: unsaturate_icon ... \n");
+
     unsaturate_icon (view_p);    
 
     
     population_p = 
 	    (population_t *)rodent_find_in_labels (view_p, event->x, event->y);
     if (!population_p) {
-    NOOP(stderr, "rodent_signal_on_motion: tip_function FALSE ... \n");
+        NOOP( "rodent_signal_on_motion: tip_function FALSE ... \n");
 	unsaturate_label (view_p);    
 	rodent_hide_tip();
 #if 10
@@ -523,6 +547,7 @@ rodent_signal_drag_data (GtkWidget * widget,
 		  guint info, 
 		  guint time, 
 		  gpointer data) {
+    fprintf(stderr,"rodent_signal_drag_data...\n");
     NOOP ("rodent_mouse: DND>> rodent_signal_drag_data\n");
 
     const population_t *population_p;
@@ -599,6 +624,7 @@ rodent_signal_drag_data (GtkWidget * widget,
 
 void
 rodent_signal_drag_leave (GtkWidget * widget, GdkDragContext * drag_context, guint time, gpointer data) {
+    fprintf(stderr,"rodent_signal_drag_leave\n");
     NOOP ("rodent_mouse: DND>> rodent_signal_drag_leave\n");
 
 }
@@ -612,6 +638,7 @@ rodent_signal_drag_delete (GtkWidget * widget, GdkDragContext * context, gpointe
 gboolean
 rodent_signal_drag_motion (GtkWidget * widget, 
 	GdkDragContext * dc, gint x, gint y, guint t, gpointer data) {
+    fprintf(stderr,"* rodent_signal_drag_motion\n");
     NOOP ("rodent_mouse: DND>> rodent_signal_drag_motion\n");
 
     gboolean target_ok = FALSE;
@@ -657,6 +684,7 @@ rodent_signal_drag_motion (GtkWidget * widget,
     }
     
     if(view_p->mouse_event.saturated_p != population_p) {
+        NOOP( "condition 3, unsaturate icon\n");
 	unsaturate_icon (view_p);
     }
     if (target_ok) {
@@ -706,6 +734,7 @@ rodent_signal_drag_motion (GtkWidget * widget,
 void
 rodent_signal_drag_data_get (GtkWidget * widget,
                       GdkDragContext * context, GtkSelectionData * selection_data, guint info, guint time, gpointer data) {
+    fprintf(stderr,"rodent_signal_drag_data_get\n");
     NOOP ("rodent_mouse: DND>> rodent_signal_drag_data_get\n");
     view_t *view_p = (view_t *) data;
     rodent_hide_tip ();
@@ -715,6 +744,7 @@ rodent_signal_drag_data_get (GtkWidget * widget,
 
 void
 rodent_signal_drag_begin (GtkWidget * widget, GdkDragContext * drag_context, gpointer data) {
+    fprintf(stderr,"rodent_signal_drag_begin\n");
     view_t *view_p = (view_t *) data;
     drag_view_p = view_p;
     rodent_hide_tip ();
@@ -725,6 +755,7 @@ rodent_signal_drag_begin (GtkWidget * widget, GdkDragContext * drag_context, gpo
 
 void
 rodent_signal_drag_end (GtkWidget * widget, GdkDragContext * context, gpointer data) {
+    fprintf(stderr,"rodent_signal_drag_end\n");
     view_t *view_p = (view_t *) data;
     widgets_t *widgets_p = &(view_p->widgets);
     //rfm_diagnostics(widgets_p, "xffm_tag/red","rodent_mouse: DND>> rodent_signal_drag_end\n", NULL); 
@@ -754,6 +785,7 @@ rodent_signal_drag_end (GtkWidget * widget, GdkDragContext * context, gpointer d
 
 void
 rodent_create_target_list (view_t * view_p) {
+    fprintf(stderr,"rodent_create_target_list\n");
     if(view_p->mouse_event.target_list)
         return;
     view_p->mouse_event.target_list = gtk_target_list_new (target_table, NUM_TARGETS);
