@@ -146,7 +146,7 @@ rodent_update_cut_icons (view_t * view_p) {
     // 1. get pasteboard status
     int pasteboard_status = rfm_pasteboard_status (view_p);
     if (!pasteboard_status) {
-        DBG("rodent_update_cut_icons():done !pasteboard_status\n");
+        TRACE("rodent_update_cut_icons():done !pasteboard_status\n");
         goto done;
     }
 
@@ -154,7 +154,7 @@ rodent_update_cut_icons (view_t * view_p) {
     // 2. get pasteboard vector
     gchar **pasteboard_v = rfm_pasteboard_v(view_p);
     if (!pasteboard_v) {
-        DBG("rodent_update_cut_icons():done !pasteboard_v\n");
+        TRACE("rodent_update_cut_icons():done !pasteboard_v\n");
         goto done;
     }
     
@@ -328,8 +328,7 @@ rodent_load_keybindings(void){
     }
     static time_t mtime=0;
     struct stat st;
-    stat(keybindings_file, &st);
-    if (st.st_mtime != mtime){
+    if (stat(keybindings_file, &st)==0 && st.st_mtime != mtime){
 	//reload
         DBG("Loading user defined keybindings...\n");
 	glib_parser(keybindings_file);
@@ -451,17 +450,6 @@ set_view_title_f (gpointer data) {
         path = g_get_host_name ();
     }
 
-#if 0
-    // deprecated: not used anymore
-    if(view_p->module) {
-        if(view_p->module && rfm_natural (PLUGIN_DIR, view_p->module, view_p->en, "window_title")) {
-            path = rfm_natural (PLUGIN_DIR, view_p->module, view_p->en, "window_title");
-        } else {
-	    NOOP("module has no window_title function\n");
-	}
-    }
-#endif
-
     rfm_global_t *rfm_global_p = rfm_global();
     set_icon_name (rfm_global_p->window, path);
     gchar *utf_tabname=rodent_get_tabname(path);
@@ -560,7 +548,7 @@ rodent_refresh (widgets_t * widgets_p, record_entry_t * en) {
 	if (valid_view && valid_entry) {
 	    gboolean same_inode =  view_p->en->st->st_ino == en->st->st_ino &&
 			       view_p->en->st->st_dev == en->st->st_dev;
-	    gboolean redundant_condition= strcmp(view_p->en->path, en->path)!=0;
+	    gboolean redundant_condition= en->path && strcmp(view_p->en->path, en->path)!=0;
 	    if (same_inode && redundant_condition){
 		// this condition also is true if a reload is attempted while the
 		// original load has not completed.
@@ -744,6 +732,7 @@ rodent_new_view(void){
 
     view_p->mouse_event.boxX = -1;
     view_p->mouse_event.boxY = -1;
+    view_p->mouse_event.drag_action = GDK_ACTION_DEFAULT;
 
     view_p->flags.preferences = __SHOW_IMAGES;
     view_p->flags.sortcolumn = GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID;
