@@ -272,6 +272,8 @@ rodent_reset_menu (void) {
 
     if(view_p->flags.preferences & SORT_ASCENDING){
 	HIDE_IT ( "ascending1");
+        // False positive.
+        // coverity[copy_paste_error : FALSE]
 	SHOW_IT ( "descending1");
     } else {
 	HIDE_IT ( "descending1");
@@ -462,10 +464,14 @@ rodent_recover_flags (extra_key_t * extra_key_p) {
     gs = g_string_new (extra_key_p->response);
     sprintf ((char *)DBH_KEY (runflags), "%10u", g_string_hash (gs));
     g_string_free (gs, TRUE);
+
+    if (dbh_load (runflags)){
     flags = (gint64 *)runflags->data;
-    dbh_load (runflags);
-    extra_key_p->flag1 = flags[0];
-    extra_key_p->flag2 = flags[1];
+        extra_key_p->flag1 = flags[0];
+        extra_key_p->flag2 = flags[1];
+    } else {
+        extra_key_p->flag1 = extra_key_p->flag2 = 0;
+    }
     dbh_close (runflags);
     pthread_mutex_unlock(&flags_mutex);
 
