@@ -184,22 +184,27 @@ rodent_tip_function (
 		     ||
 		     g_object_get_data(G_OBJECT(tip_event.tooltip_window), "normal_entry") != en);
 	     if (create) {
-		GtkWindow *old_window = tip_event.tooltip_window;
 		gchar *basename = NULL;
 		if (g_path_is_absolute(en->path)) {
                     gchar *g = g_path_get_basename(en->path);
-                    basename = g_markup_escape_text(g, -1);
+                    gchar *v = rfm_utf_string(g);
+                    basename = g_markup_escape_text(v, -1);
                     g_free(g);
-                } else basename = g_markup_escape_text(en->path, -1);
+                    g_free(v);
+                } else {
+                    gchar *v = rfm_utf_string(en->path);
+                    basename = g_markup_escape_text(en->path, -1);
+                    g_free(v);
+                }
 
 		
 		tip_event.tooltip_window = 
 		    GTK_WINDOW(rfm_create_tooltip_window(rfm_global_p->window, 
+                            GTK_WIDGET(tip_event.tooltip_window),
 			    tip_event.tooltip_pixbuf, 
 			    tip_event.tooltip_text,
 			    basename));
 		g_free(basename);
-		if(old_window) gtk_widget_destroy(GTK_WIDGET(old_window));
 		g_object_set_data(G_OBJECT(tip_event.tooltip_window), 
 			"normal_entry", en);
 	     } 
@@ -226,13 +231,12 @@ rodent_tip_function (
 		gtk_image_set_from_pixbuf (GTK_IMAGE (image), tip_event.tooltip_pixbuf);
 	    } else {
 		TRACE("creating new tip window...\n");
-		GtkWindow *old_window = tip_event.tooltip_window;
 		tip_event.tooltip_window = 
 		    GTK_WINDOW(rfm_create_tooltip_window(rfm_global_p->window, 
+                            GTK_WIDGET(tip_event.tooltip_window),
 			    tip_event.tooltip_pixbuf, 
 			    NULL,
 			    NULL));
-		if (old_window) gtk_widget_destroy(GTK_WIDGET(old_window));
 	    }
         }
 	NOOP(stderr, "tip_function TRUE ... \n");
@@ -340,7 +344,9 @@ path_info (const population_t *population_p, record_entry_t *en, const gchar *wa
         gchar lpath[_POSIX_PATH_MAX + 1];
         memset (lpath, 0, _POSIX_PATH_MAX + 1);
         if(readlink (en->path, lpath, _POSIX_PATH_MAX) > 0) {
-            gchar *escaped_markup = g_markup_escape_text(lpath, -1);
+            gchar *v = rfm_utf_string(lpath);
+            gchar *escaped_markup = g_markup_escape_text(v, -1);
+            g_free(v);
             gchar *q = rfm_utf_string (escaped_markup);
             g_free(escaped_markup);
             gchar *linkto=g_strdup_printf (_("Link to %s"), q);
@@ -427,7 +433,9 @@ path_info (const population_t *population_p, record_entry_t *en, const gchar *wa
         //    gchar *ss= rfm_time_to_string(en->st->st_mtime);   
 
         gchar *t = g_path_get_dirname (en->path);
-        gchar *escaped_markup = g_markup_escape_text(t, -1);
+        gchar *v = rfm_utf_string(t);
+        gchar *escaped_markup = g_markup_escape_text(v, -1);
+        g_free(v);
         gchar *dirname = rfm_utf_string (escaped_markup);
         g_free(t);
         g_free(escaped_markup);
@@ -558,14 +566,22 @@ rodent_get_tip_text (view_t * view_p, const population_t * population_p, record_
 	}
     } else if (view_p->en == NULL  
 	    && en && !en->module) {
-	if (en->tag) g = g_markup_escape_text(en->tag, -1);
-	else g = g_markup_escape_text(en->path, -1);
+        gchar *v = NULL;
+	if (en->tag) {
+            v = rfm_utf_string(en->tag);
+        } else {
+            v = rfm_utf_string(en->path);
+        }
+        g = g_markup_escape_text(v, -1);
+        g_free(v);
     }
 
 
     // modules can prepend text to the tooltip
     if(prepend_txt) {
-        gchar *pt = g_markup_escape_text(prepend_txt, -1);
+        gchar *v = rfm_utf_string(prepend_txt);
+        gchar *pt = g_markup_escape_text(v, -1);
+        g_free(v);
         gchar *gg = g_strconcat (prepend_txt, "\n", (g)?g:"", NULL);
         g_free (pt);
         g_free (g);
@@ -709,23 +725,22 @@ set_tooltip_info(view_t *view_p, const gchar *text, const record_entry_t *en, Gd
     tip_event.tooltip_pixbuf = tip_pixbuf; 
 
     tip_event.tooltip_active = TRUE;
-    gchar *path = g_strdup(tip_event.tooltip_entry->path);
+    const gchar *path = (tip_event.tooltip_entry)?(tip_event.tooltip_entry->path):NULL;
     g_object_set_data(G_OBJECT(rfm_global_p->window), 
 	    "tip_widgets_p", rfm_get_widget("widgets_p"));
     g_mutex_unlock(tip_mutex);
 
     g_free (oldtext);
-	population_t **population_pp = view_p->population_pp;
-	for (; population_pp && *population_pp; population_pp++){
-	    if (!(*population_pp)->en || !(*population_pp)->en->path) continue;
-	    if (strcmp((*population_pp)->en->path, path)==0){
-		(*population_pp)->flags &= (POPULATION_TIP_BUSY ^ 0xffffffff);
-		break;
-	    }
-	}
-	rfm_population_read_unlock(view_p, "set_tooltip_info");
+    population_t **population_pp = view_p->population_pp;
+    for (; population_pp && *population_pp; population_pp++){
+        if (!(*population_pp)->en || !(*population_pp)->en->path) continue;
+        if (path && strcmp((*population_pp)->en->path, path)==0){
+            (*population_pp)->flags &= (POPULATION_TIP_BUSY ^ 0xffffffff);
+            break;
+        }
+    }
+    rfm_population_read_unlock(view_p, "set_tooltip_info");
     
-    g_free(path);	
     rfm_destroy_entry(olden);
     if (oldpixbuf && G_IS_OBJECT(oldpixbuf)) g_object_unref(oldpixbuf);
 
@@ -773,9 +788,7 @@ normal_tip(const population_t * population_p){
 	const gchar *icon_id = population_p->icon_id;
 	if (!icon_id && population_p->en) icon_id = population_p->en->mimetype;
 	if (!icon_id){
-	    gchar *iconpath = ICON_get_filename_from_basename("rodent");
-	    tip_pixbuf = rfm_get_pixbuf (iconpath, BIG_ICON_SIZE); //refs
-	    g_free(iconpath);
+	    tip_pixbuf = rfm_get_pixbuf ("rodent", BIG_ICON_SIZE); //refs
 	} else {
 	    tip_pixbuf = rfm_get_pixbuf (icon_id, BIG_ICON_SIZE); //refs
 	}
@@ -893,12 +906,6 @@ rodent_activate_tip_f(view_t *view_p, void *data){
 
 
 
-
-    // This would be redundant...
-    /*
-    gboolean label_tips = (getenv ("RFM_ENABLE_LABEL_TIPS")!=NULL && strlen (getenv ("RFM_ENABLE_LABEL_TIPS")));
-    gboolean icon_tips = (getenv ("RFM_ENABLE_TIPS")!=NULL && strlen (getenv ("RFM_ENABLE_TIPS")));
-    */
     NOOP(stderr, "TIP: activate_tip_thread: 0x%x: %s\n", GPOINTER_TO_INT(g_thread_self()),
 	    activate_tip_p->thread_population_p->en->path);
 
@@ -909,7 +916,6 @@ rodent_activate_tip_f(view_t *view_p, void *data){
 	    trigger_tooltip();
 	}
     } 
-    //else if (icon_tips) {
     else {
 	NOOP(stderr, "preview tip\n");
 	 // is the work already done?
